from javaman.connexio import JManCon


class Articles:
    __slots__ = '_con'

    _url_get_articles = '/articles'
    _url_get_atributs = '/articles/{article_id}/articles_atributs'
    _url_get_articles_lots = '/articles_lots?article_atribut_id={article_atribut_id}'
    _url_get_article_lot = '/articles_lots?lot={lot}&article_atribut_id={article_atribut_id}'
    _url_get_article_lot_ex = '/articles_lots/ex?lot={lot}&article_atribut_id={article_atribut_id}'
    _url_get_article_lot_id = '/articles_lots/{id}'
    _url_get_article_estoc_magatzem = '/articles_estocs/{article_atribut_id}?magatzem_id={magatzem_id}'
    _url_post_article_lot = '/articles_lots'

    def __init__(self, con: JManCon):
        self._con = con

    def get_article(self, p_article: int):
        req = self._con.get(url=self._url_get_articles+'/'+str(p_article))
        return req.json()

    def get_article_by_ean(self, p_ean13_article: str):
        req = self._con.get(url=self._url_get_articles+'?ean13_article='+str(p_ean13_article))
        return req.json()

    def get_article_atribut(self, p_article_id: int):
        tmp_atributs = self.get_articles_atributs(p_article_id)
        res = None
        if len(tmp_atributs) > 0:
            res = tmp_atributs[0]
        return res

    def get_articles_atributs(self, p_article_id: int):
        tmp_url = Articles._url_get_atributs.format(article_id=p_article_id)
        req = self._con.get(url=tmp_url)
        return req.json()

    def get_article_lots(self, p_article_atribut_id: int, p_magatzem_id: int = None, p_palet_complert: bool = None):
        tmp_url = Articles._url_get_articles_lots.format(article_atribut_id=p_article_atribut_id)
        if p_palet_complert is not None:
            if p_palet_complert is True:
                tmp_url += "&palet_complert=S"
            else:
                tmp_url += "&palet_complert=N"
        if p_magatzem_id is not None:
            tmp_url += "&magatzem_id="+str(p_magatzem_id)

        req = self._con.get(url=tmp_url)
        return req.json()

    def get_article_lot_estoc(self, p_article_atribut_id: int, p_lot: str, p_magatzem_id: int = None, p_palet_complert: bool = None):
        tmp_url = Articles._url_get_article_lot_ex.format(article_atribut_id=p_article_atribut_id, lot=p_lot)
        if p_palet_complert is not None:
            if p_palet_complert is True:
                tmp_url += "&palet_complert=S"
            else:
                tmp_url += "&palet_complert=N"
        if p_magatzem_id is not None:
            tmp_url += "&magatzem_id="+str(p_magatzem_id)

        req = self._con.get(url=tmp_url)

        result = None
        if req is not None:
            llista = req.json()
            if len(llista) == 1:
                result = llista[0]
        return result

    def get_article_lot(self, p_article_atribut_id: int, p_lot: str):
        tmp_url = Articles._url_get_article_lot.format(article_atribut_id=p_article_atribut_id, lot=p_lot)
        req = self._con.get(url=tmp_url)
        result = None
        if req is not None:
            llista = req.json()
            if len(llista) == 1:
                result = llista[0]
        return result

    def get_article_lot_id(self, p_article_atribut_id: int, p_lot_id: int, p_magatzem_id: int = None, p_palet_complert: bool = None):
        tmp_url = Articles._url_get_article_lot_id.format(article_atribut_id=p_article_atribut_id, id=p_lot_id)
        if p_palet_complert is not None:
            if p_palet_complert is True:
                tmp_url += "&palet_complert=S"
            else:
                tmp_url += "&palet_complert=N"
        if p_magatzem_id is not None:
            tmp_url += "&magatzem_id="+str(p_magatzem_id)
        req = self._con.get(url=tmp_url)
        return req.json()

    def get_article_estoc_magatzem(self, p_article_atribut: int, p_magatzem: int):
        tmp_url = Articles._url_get_article_estoc_magatzem.format(
            article_atribut_id=p_article_atribut, magatzem_id=p_magatzem
        )
        req = self._con.get(url=tmp_url)
        llista = None
        if req is not None:
            llista = req.json()
        return llista

    def post_article_lot(self, p_data: dict):
        req = self._con.post(url=Articles._url_post_article_lot, data=p_data)
        return req.json()



from javaman.connexio import JManCon
import requests


class Pobles:
    __slots__ = '_con'

    _url_pobles = '/pobles'

    def __init__(self, con: JManCon):
        self._con = con

    def list(self):
        res = self._con.get(url=self._url_pobles)
        return res.json()

    def get(self, poble_id: int):
        res = self._con.get(url=self._url_pobles + '/' + str(poble_id))
        return res.json()

    def crear(self, poble: dict):
        res = self._con.post(url=self._url_pobles, data=poble)
        return res.json()

    def troba(self, provincia_id: int, poble: str):
        result = None
        tmp_poble = requests.utils.requote_uri(poble)
        tmp_url = self._url_pobles + '?poble_nom=' + tmp_poble + '&provincia_id=' + str(provincia_id)
        response = self._con.get(url=tmp_url)
        if response.status_code == 200:
            try:
                result = response.json()
            except Exception:
                result = None
        return result



from javaman.connexio import JManCon


class Repartidors:
    __slots__ = '_con'

    _url_get_repartior = '/repartidors/{id_rep}'

    def __init__(self, con: JManCon):
        self._con = con

    def get_repartidor(self, p_repartidor: int):
        req = self._con.get(url=Repartidors._url_get_repartior.format(id_rep=p_repartidor))
        return req.json()




from javaman.connexio import JManCon
from javaman.errors import JManErrorNoContent


class Wms:
    __slots__ = '_con'

    _url_estoc = '/wms/estoc?en_transicio=false&article_atribut_id={art_atrib_id}'  # &article_lot_id={art_lot_id}
    _url_get_palet_by_numero = "/unitats_logistiques/?numero_unitat={numero_unitat}"
    _url_get_palet_by_id = "/unitats_logistiques/ex/{unitat_logistica_id}"
    _url_post_palet = "/unitats_logistiques/ex"
    _url_put_palet = "/unitats_logistiques/ex/{unitat_logistica_id}"
    _url_delete_magatzem_ordre = '/wms/cancela_ordre/{magatzem_ordre_id}'
    _url_porta_palet = '/wms/palet_fora'
    _url_envia_palet = '/wms/palet_dins'
    _url_get_magatzem_ordre = '/magatzems_ordres/ex/{magatzem_ordre_id}'
    _url_get_magatzems_ordres = '/magatzems_ordres'
    _url_magatzem_ordre_finalitza = '/magatzems_ordres/{magatzem_ordre_id}/finalitza'
    _url_numero_unitat_logistica = '/comptadors/numero_unitat_logistica'
    _url_troba_magatzem_ordre = '/magatzems_ordres?numero_unitat_logistica={sscc}&' \
                                'magatzem_entrada_id={magatzem_entrada_id}&finalitzades=false'
    _url_afegir_unitat_logistica_albara = '/wms/albarans/{albara_id}/expedicions'
    _url_borrar_unitat_logistica_albara = '/wms/albarans/{albara_id}/expedicions/{unitat_logistica_id}'
    _url_tip_estats_unitats_logistiques = '/tip_estats_unitats_logistiques'
    _url_tip_unitats_logistiques = '/tip_unitats_logistiques'
    _url_tip_unitat_logistica_palet = '/tip_unitats_logistiques?palet=S'
    _url_unitat_logistica_format = '/unitats_logistiques_formats?tip_unitat_logistica_id={tip_unitat_logistica_id}'

    def __init__(self, con: JManCon):
        self._con = con

    def get_tip_estats_unitats_logistiques(self):
        tmp_url = Wms._url_tip_estats_unitats_logistiques
        res = self._con.get(url=tmp_url)
        return res.json()

    def get_tip_unitats_logistiques(self):
        tmp_url = Wms._url_tip_unitats_logistiques
        res = self._con.get(url=tmp_url)
        return res.json()

    def get_tip_unitat_logistica_palet(self):
        tmp_url = Wms._url_tip_unitat_logistica_palet
        result = self._con.get(url=tmp_url)
        res = None
        llista = {}
        if result is not None:
            llista = result.json()
        if (llista is not None) and (len(llista) > 0):
            res = llista[0]
        return res

    def get_unitats_logistiques_formats(self, p_tip_unitat_logistica_id: int):
        tmp_url = Wms._url_unitat_logistica_format.format(tip_unitat_logistica_id=p_tip_unitat_logistica_id)
        res = self._con.get(url=tmp_url)
        return res.json()

    def get_estoc(self, p_article_atribut_id: int, p_article_lot_id: int = None):
        tmp_url = Wms._url_estoc.format(art_atrib_id=p_article_atribut_id)
        if p_article_lot_id is not None:
            tmp_url = tmp_url.format(art_lot_id=p_article_lot_id)
        res = self._con.get(url=tmp_url)
        return res.json()

    def get_palet_by_numero(self, p_numero_palet: str):
        tmp_url = Wms._url_get_palet_by_numero.format(numero_unitat=p_numero_palet)
        res = self._con.get(url=tmp_url).json()
        if len(res) == 0:
            raise JManErrorNoContent()
        return res[0]

    def get_palet_by_id(self, p_unitat_logistica_id: int):
        tmp_url = Wms._url_get_palet_by_id.format(unitat_logistica_id=p_unitat_logistica_id)
        res = self._con.get(url=tmp_url)
        return res.json()

    def delete_magatzem_ordre(self, p_magatzem_ordre_id: int):
        tmp_url = Wms._url_delete_magatzem_ordre.format(magatzem_ordre_id=p_magatzem_ordre_id)
        self._con.delete(url=tmp_url)
        return

    def porta_palet(self, p_magatzem_desti_id: int,
                    p_unitat_logistica_id: int = None,
                    p_numero_unitat: str = None,
                    p_article_atribut_id: int = None,
                    p_article_lot_id: int = None):
        parametres = {"unitat_logistica_id": p_unitat_logistica_id,
                      "numero_unitat_logistica": p_numero_unitat,
                      "article_atribut_id": p_article_atribut_id,
                      "article_lot_id": p_article_lot_id,
                      "magatzem_origen_id": None,
                      "magatzem_desti_id": p_magatzem_desti_id,
                      "iniciar_ordre_auto": True
                      }
        res = self._con.post(url=Wms._url_porta_palet, data=parametres)
        return res.json()

    def crea_palet(self, p_data: dict):
        req = self._con.post(url=self._url_post_palet, data=p_data)
        return req.json()

    def vincula_unitat_logistica_albara(self, p_albara_id: int, p_unitat_logistica: dict):
        req = self._con.post(
            url=self._url_afegir_unitat_logistica_albara.format(albara_id=p_albara_id), data=p_unitat_logistica)
        return req.json()

    def desvincula_unitat_logistica_albara(self, p_albara_id: int, p_unitat_logistica_id: dict):
        req = self._con.delete(
            url=self._url_borrar_unitat_logistica_albara.format(
                albara_id=p_albara_id, unitat_logistica_id=p_unitat_logistica_id)
        )
        return req.json()

    def modifica_palet(self, p_palet_id: int, p_data: dict):
        req = self._con.put(url=self._url_put_palet.format(unitat_logistica_id=p_palet_id), data=p_data)
        return req.json()

    def troba_magatzem_ordre(self, p_sscc: str):
        req = self._con.get(url=self._url_troba_magatzem_ordre.format(
            sscc=p_sscc, magatzem_entrada_id=self._con.config.magatzem_gdis_id())
        )
        return req.json()

    def envia_palet(self, p_magatzem_origen_id: int,
                    p_magatzem_desti_id: int,
                    p_unitat_logistica_id: int):

        parametres = {"unitat_logistica_id": p_unitat_logistica_id,
                      "article_atribut_id": None,
                      "article_lot_id": None,
                      "magatzem_origen_id": p_magatzem_origen_id,
                      "magatzem_desti_id": p_magatzem_desti_id
                      }
        res = self._con.post(url=Wms._url_envia_palet, data=parametres)
        return res.json()

    def get_magatzem_ordre(self, p_magatzem_ordre_id: int):
        res = self._con.get(url=Wms._url_get_magatzem_ordre.format(magatzem_ordre_id=p_magatzem_ordre_id))
        return res.json()

    def get_magatzem_ordre_by_sscc(self, p_sscc: str):
        res = self._con.get(url=Wms._url_get_magatzems_ordres + "?numero_unitat_logistica="+p_sscc+"&finalitzades=False")
        return res.json()

    def nou_numero_unitat_logistica(self) -> str:
        res = self._con.post(url=Wms._url_numero_unitat_logistica, data=None)
        return res.text

    def finalitza_magatzem_ortdre(self, p_magatzem_ordre_id: int):
        res = self._con.put(url=Wms._url_magatzem_ordre_finalitza.format(magatzem_ordre_id=p_magatzem_ordre_id), data=None)
        return res.text



from javaman.connexio import JManCon


class Albarans:
    __slots__ = '_con'

    _url_get_albara_imprimir = '/albarans/{albara_id}/imprimir'

    def __init__(self, con: JManCon):
        self._con = con

    def get_albara_imprimir(self, p_albara_id: int):
        req = self._con.get(url=self._url_get_albara_imprimir.format(albara_id=p_albara_id))
        return req.json()



from javaman.connexio import JManCon


class Regularitzacions:
    __slots__ = '_con'

    _url_get_regularitzacio = "/regularitzacions/ex/{regularitzacio_id}"
    _url_post_regularitzacio = "/regularitzacions/ex"

    def __init__(self, con: JManCon):
        self._con = con

    def get_regularitzacio(self, p_regularitzacio_id: int):
        tmp_url = Regularitzacions._url_get_regularitzacio.format(regularitzacio_id=p_regularitzacio_id)
        req = self._con.get(url=tmp_url)
        return req.json()

    def post_regularitzacio(self, p_data: dict):
        req = self._con.post(url=self._url_post_regularitzacio, data=p_data)
        return req.json()



from .albarans import Albarans
from .articles import Articles
from .clients import Clients
from .tercers import Tercers
from .repartidors import Repartidors
from .comandes import Comandes
from .pobles import Pobles
from .provincies import Provincies
from .portals_web import PortalsWeb
from .regularitzacions import Regularitzacions
from .usuaris import Usuaris
from .wms import Wms
from .magatzems import Magatzems



class JManError(Exception):
    pass


class JManErrorUnauthorized(JManError):
    pass


class JManErrorClient(JManError):
    pass


class JManErrorConnection(JManError):
    pass


class JManErrorApp(JManError):
    pass


class JManErrorNoContent(JManError):
    pass



from javaman.connexio import JManCon


class Clients:
    __slots__ = '_con'

    _url_crear_client = '/clients'
    _url_get_client = '/clients'
    _url_importar_tercers = '/importar_tercers'

    def __init__(self, con: JManCon):
        self._con = con

    def crear(self, p_client: dict):
        req = self._con.post(url=self._url_crear_client, data=p_client)
        return req.json()

    def get_client(self, p_client: int):
        req = self._con.get(url=self._url_crear_client+'/'+str(p_client))
        return req.json()

    def importar_tercers(self, p_dades: dict):
        req = self._con.post(url=self._url_importar_tercers, data=p_dades)
        return req.json()



from javaman.connexio import JManCon


class Comandes:
    __slots__ = '_con'

    _url_portal_web_comanda = '/portals_web/{numcom}/comanda'
    _url_pendent_servir = '/comandes/{id_com}/pendent_servir'
    _url_bloqueig = '/comandes/{id_com}/magatzem_retingut'
    _url_comencar = '/comandes/{id_com}/iniciar'
    _url_cancelar = '/comandes/{id_com}/cancelar'
    _url_tancar = '/comandes/{id_com}/tancar'
    _url_albarans = '/comandes/{id_com}/albarans/iniciats'
    _url_comanda_distribucio = '/comandes/{id_com}/distribucio'

    def __init__(self, con: JManCon):
        self._con = con

    def crear(self, portal_web_token: str, comanda: dict):
        tmp_url = Comandes._url_portal_web_comanda.format(numcom=portal_web_token)
        req = self._con.post(url=tmp_url, data=comanda)
        return req.json()

    def bloquejar(self, comanda_id: int):
        req = self._con.post(url=Comandes._url_bloqueig.format(id_com=comanda_id), data=None)
        return req.json()

    def pendent_servir(self, comanda_id: int):
        req = self._con.post(url=Comandes._url_pendent_servir.format(id_com=comanda_id), data=None)
        return req.json()

    def comencar(self, comanda_id: int):
        req = self._con.post(url=Comandes._url_comencar.format(id_com=comanda_id), data=None)
        return req.json()

    def llistar_albarans(self, comanda_id: int):
        req = self._con.get(url=Comandes._url_albarans.format(id_com=comanda_id))
        return req.json()

    def assignar_usuari(self, comanda_id: int, emplat_id: int):
        req = self._con.post(url='/comandes/' + str(comanda_id) + '/assignar', data={'id': emplat_id})
        return req.json()

    def pausar(self, comanda_id: int):
        req = self._con.post(url='/comandes/' + str(comanda_id) + '/pausar', data=None)
        return req.json()

    def cancelar(self, comanda_id: int):
        req = self._con.post(url=Comandes._url_cancelar.format(id_com=comanda_id))
        return req.json()

    def tancar(self, comanda_id: int):
        req = self._con.post(url=Comandes._url_tancar.format(id_com=comanda_id))
        return req.json()

    def put_comanda_distribucio(self, comanda_distribucio: dict):
        tmp_url = Comandes._url_comanda_distribucio.format(id_com=comanda_distribucio["comanda_id"])
        req = self._con.put(url=tmp_url, data=comanda_distribucio)
        return req.json()



from javaman.connexio import JManCon
import requests


class Magatzems:
    __slots__ = '_con'

    _url_magatzems = '/magatzems'
    _url_tip_magatzems = '/tip_magatzems'
    _url_moviment_magatzem_ex = '/moviments_magatzems/ex'

    def __init__(self, con: JManCon):
        self._con = con

    def list_magatzems(self):
        res = self._con.get(url=self._url_magatzems)
        return res.json()

    def get_magatzem(self, magatzem_id: int):
        res = self._con.get(url=self._url_magatzems + '/' + str(magatzem_id))
        return res.json()

    def list_tip_magatzems(self):
        res = self._con.get(url=self._url_tip_magatzems)
        return res.json()

    def get_moviment_magatzem_ex(self, moviment_magatzem_id: int):
        res = self._con.get(url=self._url_moviment_magatzem_ex + '/' + str(moviment_magatzem_id))
        return res.json()
